#include <MT/array.h>
#include <math.h>
#include <fstream>

template<class T> void save(const T& x,const char *filename){
    std::ofstream os(filename);
    x.writeRaw(os);
    os <<std::endl;
}

int main(int argn,char** argv){
  uint T=1000,K=2;
  double rho=.01,eps=.2;
  uint i,t;

  //-- build transition and observation matrices
  arr P0(K),A(K,K),B(K,K);
  A=rho;  for(i=0;i<K;i++) A(i,i) = 1.-rho;
  B=eps;  for(i=0;i<K;i++) B(i,i) = 1.-eps;
  P0=1./K;
  save(A,"z.A");
  save(B,"z.B");

  //-- generate a sample from the HMM
  arr p(K),At;
  uintA x(T),y(T);
  transpose(At,A);
  p=P0;
  for(t=0;t<T;t++){
    x(t) = SUS(p); //state
    y(t) = SUS(B[x(t)]); //observation
    p = A[x(t)];
  }
  x.reshape(T,1);  save(x,"z.x");  x.reshape(T);
  y.reshape(T,1);  save(y,"z.y");  y.reshape(T);
  
  //-- evidences from observations
  arr phi(T,K);
  for(t=0;t<T;t++) phi[t]=B[y(t)];
  save(phi,"z.phi");

  //-- fwd & bwd messages
  arr a(T,K),b(T,K);
  a[0]()=P0;
  for(t=1;t<T;t++) a[t]() = A*(phi[t-1]%a[t-1]); // %=element-wise multiplication, *=inner product
  b[T-1]()=1.;
  for(t=T-1;t--;)  b[t]() = (phi[t+1]%b[t+1])*A;
  save(a,"z.alpha");
  save(b,"z.beta");

  //-- posterior beliefs
  arr post(T,K);
  for(t=0;t<T;t++) post[t]() = a[t]%phi[t]%b[t];

  //-- normalize posterior
  double Z=sum(post[0]);
  for(t=0;t<T;t++) post[t]()/=Z;
  
  save(post,"z.post");
  //fprintf(gp,"load 'plt'\n");
}
